<?xml version="1.0"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:xhtml="http://www.w3.org/1999/xhtml"
>
	<xsl:output method = "xml" encoding="UTF-8" />
	<!-- 
		It would be nice if there was a dtd or an overview of 
		the drupal-xml-import-export format somewhere.
		As it is, I'll just black-box it and emulate from example.

		Embedded HTML should be character-escaped ...


		To find the nodes by local name without namespaces handicapping me,
		Input html must either have no namespace ( select="html" )
		or be in the xhtml namespace ( select="xhtml:html" ).
		Use these two patterns at all times when matching nodes by name :-(

		... This may be avoided slightly by adjusting the input namespace 
		during pre-processing ... although the syntax will still require 
		the prefix - a doc with any namespace requires the prefix to
		be used from then on.

		Due to namespaces again, if embedding unescaped html into the resulting xmlnode format,
		the nodes will be tagged with messy xmlns= tags. THIS IS CORRECT
		as we are not a true html document. BUT to turn this behaviour off,
		you can set xmlns="http://www.w3.org/1999/xhtml" in THIS file header.
		It's a lie, but it results in cleaner (albeit hybrid) code.
		
		The xsl:text nodes are just scattered around to force some pretty-printing on the output

		.dan.
	-->
	
	<xsl:template match="/">	
		<nodes>
		<xsl:apply-templates select="html|xhtml:html" />
		</nodes>
	</xsl:template>

	<xsl:template match="html|xhtml:html">	
		<node type="story">
		
		<!-- META And Header extraction -->
		<!-- 
			Unsupported/unused  potential node attributes:
			nid="3" 
			promote="1" 
			moderate="0" 
  			changed="1127211405" 
  			created="1127194744" 
  			status="1"
		-->
		
		<!-- 
			Note: TODO 
			Need to deal with the possible difference between 
			document title and H1 display title .
			Look for H1 or anything id=pagetitle first
			If that fails, use the meta title.
			Note, if the visible heading was found, it should be 
			removed from the body content so as not to show up again.
			Some exceptions to do so are defined later
		-->
	    <title>
			<xsl:choose>
				
				<xsl:when test="//h1|//xhtml:h1">
			    	<xsl:value-of select="//h1|//xhtml:h1" />
				</xsl:when>
				
				<xsl:when test="//*[@id='pagetitle']">
					<!-- 
						Nice sites set the id of their title.
						May add other #ids here as I find them
					-->
			    	<xsl:value-of select="//*[@id='pagetitle']" />
				</xsl:when>

				<xsl:when test="count(//h2|//xhtml:h2)=1">
					<!-- 
						Maybe it used an h2 because h1 was ugly.
						If there is ONLY ONE h2, use that
					-->
			    	<xsl:value-of select="//h2|//xhtml:h2" />
				</xsl:when>
				
				<xsl:otherwise>
					<!-- 
						In practice, the html title is often more verbose than we want
						and too often it's the same across whole sections, but it'll have to do.
					-->
				
			    	<xsl:value-of select="head/title|xhtml:head/xhtml:title" />
				</xsl:otherwise>
			</xsl:choose>
	    </title>

<xsl:text>
</xsl:text>


		<!-- Content extraction -->
		<teaser>
		<!-- Grab the first two paragraphs ... found anywhere in the doc -->
		
			<!-- 
				Don't use just select="//p[position()=2]" as that gets all 
				p nodes who are number 2 in their respective contexts ... 
				I need to find numbers 1 and 2 in the set of all p nodes.
			-->
			<xsl:variable name="set" select="//p|//xhtml:p"/>
			<!-- <xsl:for-each select="$set[1]|$set[2]"> OK, just one para -->
			<xsl:for-each select="$set[1]">
				<xsl:call-template name="escapeall" />
 			</xsl:for-each>

		</teaser>

<xsl:text>

</xsl:text>
		
		<body>
			<!--	Attempt to find any defined content divs. Failing that, include everything	-->
			<xsl:choose>
				<!--
					Need to detect them before using them to avoid 
					duplication of content if more than one match is valid.
					So look-see before select
				-->
				
				<xsl:when test="//*[@id='main']">
					<xsl:for-each select="//*[@id='main']/*" >
						<xsl:call-template name="escapeall" />
					</xsl:for-each>
				</xsl:when>

				<xsl:when test="//*[@id='bodyCopy']">
					<xsl:for-each select="//*[@id='bodyCopy']/*" >
						<xsl:call-template name="escapeall" />
					</xsl:for-each>
				</xsl:when>

				<xsl:when test="//*[@id='content']">
					<xsl:for-each select="//*[@id='content']/*" >
						<xsl:call-template name="escapeall" />
					</xsl:for-each>
				</xsl:when>
			  
				<xsl:when test="//*[@class='reference']">
				<!--
					This one grabs content from php.net documentation
				-->
					<xsl:for-each select="//*[@class='reference']/*" >
						<xsl:call-template name="escapeall" />
					</xsl:for-each>
				</xsl:when>

				<xsl:otherwise>
				
<xsl:text>
				
</xsl:text>
				 <xsl:comment>Imported Full Body :(</xsl:comment>
<xsl:text>

</xsl:text>
				<!-- Absolutely no magic here - dump it ALL in -->

					<xsl:for-each select="//body/*|//xhtml:body/*" >
						<xsl:call-template name="escapeall" />
					</xsl:for-each>
					

				</xsl:otherwise>
			</xsl:choose>
		</body>

<xsl:text>
</xsl:text>
		
		<user uid="1">dman</user>

<xsl:text>
</xsl:text>
		
		<vocabularies>
			<vocabulary vid="1">
				<name>Section</name>
				<terms>
						<term tid="3">
					<name>Portfolio</name>
					</term>
				</terms>
			</vocabulary>
		</vocabularies>
		
    	</node>

<xsl:text>

</xsl:text>

	</xsl:template>


	<!-- Identity transformation template - to inline existing HTML  -->			
	<xsl:template name="passthrough" match="@*|node()|text()">
	  <xsl:copy>
		<xsl:apply-templates select="@*|node()|text()"/>
	  </xsl:copy>
	</xsl:template>



	<!-- 
		Flatten all HTML or XML into escaped code - safe text for embedding without parsing
		I thought I could find a code snippet to do this at Mulberry or Pawsons FAQ, but no.
		May miss out on advanced nodes (comments)
		.dan. Sep 2005
	-->			
	<xsl:template name="escapeall">

	<xsl:choose>
		<!-- first, a couple of exceptions that are NOT to be rendered -->
		<xsl:when test="name()='h1'"><!-- skip --></xsl:when>
		<xsl:when test="@id='pagetitle'"><!-- skip --></xsl:when>

		<xsl:when test="name()">
			<!-- named nodes get the full works -->

			<xsl:text>&lt;</xsl:text>
			<xsl:value-of select="name()" />
			<xsl:text> </xsl:text>
			<xsl:for-each select="@*" >
				<xsl:value-of select="name()" />
				<xsl:text>="</xsl:text>
				<xsl:value-of select="." />
				<xsl:text>" </xsl:text>
			</xsl:for-each>
			
			<xsl:choose>
			<xsl:when test="node()|*">
				<!-- has content... recurse -->

				<xsl:text>&gt;</xsl:text>
				<xsl:for-each select=" * | comment() | processing-instruction() | text()">
					<xsl:call-template name="escapeall" />
				</xsl:for-each>
		
				<xsl:text>&lt;/</xsl:text>
				<xsl:value-of select="name()" />
				<xsl:text>&gt;</xsl:text>

			</xsl:when>
			<xsl:otherwise>
				<xsl:text>/&gt;</xsl:text>
			</xsl:otherwise>
			</xsl:choose>
	
		</xsl:when>
		<xsl:otherwise> 
			<!-- a node with no name ... must be text -->
			<xsl:copy-of select="."/>
		</xsl:otherwise>
		</xsl:choose>


	</xsl:template>


</xsl:stylesheet>