<?xml version="1.0"?>
<xsl:stylesheet version="1.0" 
xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
xmlns:xhtml="http://www.w3.org/1999/xhtml"
xmlns="http://www.w3.org/1999/xhtml"
>

<!--
	When an XML doc is converted, it is presented to the viewer out of context.
	If the context was important (eg it was XHTML comprising a hrefs and img srcs)
	that must be repaired. 
	Using the HTML base markup is ugly. instead we'll use it as a parameter to all links.
	
	If href_base parameter is set, then all hrefs, img src etc in the document
	need to be adjusted to point back to the real data source location.
	This will have to be applied by clever re-writing
	of the links, as this template attempts.
-->
	<xsl:param name="href_base">thebase</xsl:param>
	<xsl:param name="src_base">thesrc</xsl:param>


<!--
	output must be set to be compliant, else the meta and link tags
	(unclosed singletons in old HTML) will not validate on the next pass.
-->
	<xsl:output method = "xml" encoding="UTF-8" />
	
	
<!-- 
	rewrite everything that uses 'href' 
-->
   <xsl:template name="rewritehrefs" match="node()[@href]">
		<xsl:call-template name="rewritelink">
			<xsl:with-param name="attname">href</xsl:with-param>
			<!--
				Problem - hrefs that link directly to resources (eg, the big jpeg) need extra care.
				Looking for the suffix is not great, but it's a best guess.
				Should use ends-with, but sablotron doesn't play that.
				This needs work and may cause problems.
			-->
			<xsl:with-param name="linkbase">
				<xsl:choose>
					<xsl:when test="contains(@href,'.htm') or contains(@href,'.php') or contains(@href,'.asp') or contains(@href,'.jsp') or not( contains(@href,'.') )">
						<xsl:value-of select="$href_base" />
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="$src_base" />
					</xsl:otherwise>
				</xsl:choose>
			</xsl:with-param>
			

		</xsl:call-template>
   </xsl:template>


<!-- 
	rewrite everything that uses 'src'
 -->
   <xsl:template name="rewritesrcs" match="node()[@src]">
		<xsl:call-template name="rewritelink">
			<xsl:with-param name="attname">src</xsl:with-param>
			<xsl:with-param name="linkbase"><xsl:value-of select="$src_base" /></xsl:with-param>
		</xsl:call-template>
   </xsl:template>


   <xsl:template name="rewritebackgroundlinks" match="node()[@background]">
		<xsl:call-template name="rewritelink">
			<xsl:with-param name="attname">background</xsl:with-param>
			<xsl:with-param name="linkbase"><xsl:value-of select="$src_base" /></xsl:with-param>
		</xsl:call-template>
   </xsl:template>
 
 
   <xsl:template name="rewriteembeddedmedia" match="node()[@name='movie' and name()='param']">
    	<!--
    		SWF and things with <param name='movie' value='source/file.swf' /> syntax.
    		Awkward node()[name='node-name'] xpath is used here to ignore namespaces.
    	-->
		<xsl:call-template name="rewritelink">
			<xsl:with-param name="attname">value</xsl:with-param>
			<xsl:with-param name="linkbase"><xsl:value-of select="$src_base" /></xsl:with-param>
		</xsl:call-template>
   </xsl:template>



<!-- 
	rewrite everything that has a link (href,src, etc)
	to use a link relative to somewhere else
 -->
   <xsl:template name="rewritelink">
	   <xsl:param name="attname" />
	   <xsl:param name="linkbase" />
	<xsl:copy>
		
		<xsl:for-each select="@*" >
			<xsl:choose>

				<xsl:when test="name()=$attname">
				
					<!-- these ARE the droids you are looking for -->
					<xsl:choose>

						<xsl:when test="not(contains(.,':')) and not(starts-with(.,'/'))">
					   <!-- 
					       It has no scheme, and is doesn't start with /
					       It's a partial Url, may need help being relative
					       Rewrite it
				        -->
							<xsl:attribute name="{$attname}">
							<xsl:value-of select="$linkbase" /><xsl:value-of select="." />
							</xsl:attribute>
				
						</xsl:when>
						<xsl:otherwise>
						<!-- 
							Full Url, defines scheme, leave as is 
						-->
							<xsl:copy-of select="." />
							<xsl:apply-templates select=" * | text()"/>
						</xsl:otherwise>

					</xsl:choose>
				</xsl:when>
				<xsl:otherwise>
					<!-- 
						It's one of the other attributes
						Copy all and carry on 
					-->
					<xsl:copy />
				</xsl:otherwise>
			</xsl:choose>
		</xsl:for-each>
		<!-- 
		  done attributes, needs to carry on with the contents 
		-->
			<xsl:apply-templates />

		</xsl:copy>

   </xsl:template>
	    

	    
	<!-- Identity transformation template - let everything else pass through  -->			
	<xsl:template match=" * | comment() | processing-instruction() | text()" >
		<xsl:copy>
			<xsl:copy-of select="@*"/>
			<xsl:apply-templates select=" * | comment() | processing-instruction() | text()"/>
		</xsl:copy>
	</xsl:template>	    
	
</xsl:stylesheet>